=== modified file 'libssts/DynamicArray.c'
--- libssts/DynamicArray.c	2016-01-25 21:52:28 +0000
+++ libssts/DynamicArray.c	2016-01-26 12:26:20 +0000
@@ -184,6 +184,8 @@
                              spointer data) {
   s_dbg_print ("Removing item from array");
   FreeFunc func = self->free_func;
-  func (item);
+  if (func) {
+    func (item);
+  }
 }
 

=== modified file 'libssts/Map.c'
--- libssts/Map.c	2016-01-25 21:52:28 +0000
+++ libssts/Map.c	2016-01-26 12:26:20 +0000
@@ -52,6 +52,9 @@
                                      SLinkedList * bucket,
                                      spointer key);
 
+void
+_internal_s_map_free_buckets (SLinkedList * bucket);
+
 SMapItem *
 s_map_item_new (void * key, void * value) {
   SMapItem * self = malloc (sizeof (SMapItem));
@@ -67,16 +70,16 @@
                  FreeFunc free_value) {
   s_dbg_print ("Freeing Item!");
   if (free_key) {
+    s_dbg_print ("Freeing Key");
     free_key (self->key);
   }
   if (free_value) {
+    s_dbg_print ("Freeing Value");
     free_value (self->value);
   }
   free (self);
 }
 
-void
-_internal_s_map_free_buckets (SLinkedList * self);
 
 SMap *
 s_map_new (CompFunc comp_func,
@@ -99,7 +102,7 @@
    * Creating a place to store the items.
    */
   self->array = s_dynamic_array_new (S_MAP_DEFAULT_NUMBER_OF_BUCKETS,
-                                     FREEFUNC (s_map_item_free));
+                                     FREEFUNC(_internal_s_map_free_buckets));
   
   /* if no func is set we have to use some other metod of
    */
@@ -118,8 +121,8 @@
   s_map_for_each (self,
                   FOREACHFUNC(_s_map_internal_free_map_items_for_each),
                   free_data);
-
-  s_dynamic_array_free (self->array, FALSE);
+  
+  s_dynamic_array_free (self->array, TRUE);
   free (self);
 }
 
@@ -165,17 +168,19 @@
 
 spointer
 s_map_get (SMap * self, spointer key) {
-  spointer ret_val = NULL;
+  return s_map_get_item(self, key)->value;
+}
 
+SMapItem *
+s_map_get_item (SMap * self, spointer key) {
   SLinkedList * bucket = _s_map_internal_find_bucket (self, key);
   if (bucket) {
     SMapItem * item = _s_map_internal_find_item_in_bucket (self, bucket, key);
     if (item) {
-      ret_val = item->value;
+      return item;
     }
   }
-
-  return ret_val;
+  return NULL;
 }
 
 void
@@ -257,3 +262,8 @@
   }
 }
 
+void
+_internal_s_map_free_buckets (SLinkedList * bucket) {
+  s_linked_list_free (bucket, FALSE);
+}
+

=== modified file 'libssts/Map.h'
--- libssts/Map.h	2015-11-14 17:14:26 +0000
+++ libssts/Map.h	2016-01-26 12:26:20 +0000
@@ -125,9 +125,9 @@
  */
 SMap *
 s_map_new (CompFunc comp_func,
-                  HashFunc key_hash_func,
-                  FreeFunc free_key,
-                  FreeFunc free_value);
+           HashFunc key_hash_func,
+           FreeFunc free_key,
+           FreeFunc free_value);
 
 /** @breif
  * This function frees an instance of an SMap.
@@ -159,6 +159,10 @@
 spointer
 s_map_get (SMap * self, spointer key);
 
+
+SMapItem *
+s_map_get_item (SMap * self, spointer key);
+
 /**
  * This function removes an item from 
  */

=== modified file 'libssts/hash.h'
--- libssts/hash.h	2015-11-14 17:14:26 +0000
+++ libssts/hash.h	2016-01-26 12:26:20 +0000
@@ -38,8 +38,7 @@
 /**
  * what hash function s_hash is defined to.
  */
-#define s_hash(k) sdbm_hash(k)
-
+#define s_hash sdbm_hash
 
 S_END_DECLS
 

=== modified file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2016-01-25 21:52:28 +0000
+++ tests/CMakeLists.txt	2016-01-26 12:26:20 +0000
@@ -11,6 +11,8 @@
   dynamic_array.c
   linked_list.h
   linked_list.c
+  map.h
+  map.c
   callback.h
   callback.c
   thread.h

=== modified file 'tests/main.c'
--- tests/main.c	2016-01-25 21:52:28 +0000
+++ tests/main.c	2016-01-26 12:26:20 +0000
@@ -12,6 +12,7 @@
 #include "callback.h"
 #include "linked_list.h"
 #include "thread.h"
+#include "map.h"
 
 int main  (int argc, char ** argv, char ** argp) {
   setup_suite ("SSTS");
@@ -21,6 +22,7 @@
   test_unit (test_error, "Error");
   test_unit (test_dynamic_array, "Dynamic Array");
   test_unit (test_linked_list, "Linked List");
+  test_unit (test_map, "Map");
   //test_unit (test_callback, "Callback");
   //test_unit (test_thread, "Threading");
 

=== added file 'tests/map.c'
--- tests/map.c	1970-01-01 00:00:00 +0000
+++ tests/map.c	2016-01-26 12:26:20 +0000
@@ -0,0 +1,38 @@
+#include "SimpleTypeSystem.h"
+#include "test_macros.h"
+#include <stdlib.h>
+#include <stdint.h>
+#include <time.h>
+
+void
+for_each_print_map (SLinkedList * self, SMapItem * item, sint * itt) {
+  s_print ("Key: \"%s\" Value: %d on iteration: %d\n", (schar *)item->key,
+                                                       (sint)item->value,
+                                                       (*itt));
+  (*itt)++;
+}
+
+int test_map (void) {
+  setup_unit ();
+  
+  SMap * map = NULL;
+  map =  s_map_new (COMPFUNC (s_string_is_equal),
+                    HASH_FUNC (s_hash),
+                    NULL,
+                    NULL);
+  
+  test_case (map != NULL, "Map is not null.");
+  
+  s_map_add (map, "Hello", (spointer)(sint) 1337);
+  
+  test_case ((s_map_get (map, "Hello") == (sint) 1337), "The got value is the"
+                                                  " same as the one added to"
+                                                  " the map");
+  SMapItem * item = s_map_get_item (map, "Hello");
+  s_print ("The key was: %s, the Value was: %d\n", (schar *) item->key,
+                                                   (sint) item->value);
+  
+  s_map_free (map, TRUE);
+  
+  end_unit ();
+}

=== added file 'tests/map.h'
--- tests/map.h	1970-01-01 00:00:00 +0000
+++ tests/map.h	2016-01-26 12:26:20 +0000
@@ -0,0 +1,4 @@
+#ifndef __H_TEST_MAP__
+#define __H_TEST_MAP__
+int test_map (void);
+#endif

